import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import QtQuick.Shapes
import Qt.labs.qmlmodels
import Qt5Compat.GraphicalEffects

import "components"
import "./view"

import Artix.Handlers
import Artix.Core
import Artix.Gui
import Artix.Models

/**
 * @brief Форма выбора признаков клиента/отображения информации о клиенте
 */
FullscreenDialog {
    id: root
    property var fieldsById: ({})
    property real delegateHeight: 84
    property real delegateSpacing: 22
    property real formPadding: 24
    property bool showInfo: params.fields.length === 0

    // Фильтр для работы с клавиатурой
    property DialogEventFilter eventFilter: DialogEventFilter {
        onOk: accept()
        onCancel: handler.sendAnswer({})
    }

    component InfoBoxCommon: GroupBox {
        Style.class: "customerInfoCommonData"
        interactive: false
        leftPadding: 32
        topPadding: 24
        rightPadding: 32
        bottomPadding: 24
        Layout.fillWidth: true
    }

    component InfoBoxBlue: InfoBoxCommon {
        Style.class: "customerInfoBlue"
        Layout.preferredHeight: 104
    }

    component InfoBoxGreen: InfoBoxCommon {
        Style.class: "customerInfoGreen"
        Layout.preferredHeight: 166
        interactive: false
    }

    component ChoiceDelegate: Item {
        objectName: field.fieldId
        height: delegateHeight
        width: buttons.implicitWidth
        y: fieldRow * (delegateHeight + delegateSpacing)
        visible: needVisible(field)
        property int fieldType: type
        property int choiceValue: group.checkedButton ? group.checkedButton.buttonIndex : -1
        ButtonGroup { id: group; buttons: buttons.children }
        ColumnLayout {
            spacing: 10
            BasicLabel {
                Style.class: "customerInfoLabel"
                Tr { tr: field.description }
            }
            RowLayout {
                id: buttons
                spacing: 0
                Repeater {
                    id: repeater
                    model: field.labels.count
                    delegate: BasicRadioButton {
                        Style.class: "inputMultiText"
                        checked: field.defaultValue ? index === Number(field.defaultValue) : index === 0
                        property int buttonIndex: index
                        Tr { tr: field.labels.value(index) }
                        Line {
                            visible: index !== 0
                            anchors.right: parent.left
                            height: parent.height - 15
                            anchors.verticalCenter: parent.verticalCenter
                            strokeStyle: ShapePath.DashLine
                            direction: LineDirection.Vertical
                            dashPattern: [2, 3]
                            strokeWidth: 1
                            color: Style.parameter("color.gray")
                        }
                    }
                }
            }
        }
        Component.onCompleted: root.fieldsById[objectName] = this
    }

    contentItem: RowLayout {
        // Левая часть
        Rectangle {
            id: contentRect
            Layout.fillWidth: true
            Layout.fillHeight: true
            color: Style.parameter("color.modalBackground")
            clip: true
            BasicLabel {
                id: titleLabel
                maximumLineCount: 1
                width: parent.width - formPadding
                elide: Text.ElideRight
                Style.class: "customerInfoTitle"
                anchors { top: parent.top; left: parent.left; leftMargin: 24; topMargin: 24 }
                text: params.customerInfo.firstName + " " + params.customerInfo.lastName;
            }
            Item {
                width: parent.width
                height: parent.height - titleLabel.height
                y: titleLabel.height + formPadding
                clip: true
                Flickable {
                    anchors.fill: parent
                    contentHeight: contentBox.height + delegateHeight * params.fields.length
                                   + delegateSpacing * (params.fields.length - 1) + formPadding * 2
                    interactive: contentHeight > contentRect.height
                    ColumnLayout {
                        id: contentBox
                        Layout.fillWidth: true
                        spacing: 0
                        anchors { left: parent.left; right: parent.right; top: parent.top; }
                        Item {
                            id: contentInfo
                            Layout.fillWidth: true
                            Layout.preferredHeight: (contentInfo.expand ? 802 : 268)
                            property bool expand: false
                            ColumnLayout {
                                spacing: 16
                                Layout.fillWidth: true
                                Layout.fillHeight: true
                                anchors {
                                    left: parent.left; right: parent.right; top: parent.top;
                                    leftMargin: 24; topMargin: 16; rightMargin: 24;
                                }
                                InfoBoxCommon {
                                    Layout.preferredHeight: 216
                                    contentItem: ColumnLayout {
                                        spacing: 24
                                        RowLayout {
                                            spacing: 24
                                            ColumnLayout{
                                                BasicLabel {
                                                    Style.class: "customerInfoLabel"
                                                    Tr { code: "customerNameLabel"; defaultMessage: "Организация" }
                                                }
                                                BasicLabel {
                                                    Layout.fillWidth: true
                                                    maximumLineCount: 2
                                                    Style.class: "customerInfoValue"
                                                    elide: Text.ElideRight
                                                    text: params.customerInfo.customerName
                                                }
                                            }
                                            ColumnLayout {
                                                Layout.alignment: Qt.AlignTop
                                                BasicLabel {
                                                    Style.class: "customerInfoLabel"
                                                    Tr { code: "customerCreditTypeLabel"; defaultMessage: "Тип клиента" }
                                                }
                                                BasicLabel {
                                                    Layout.preferredWidth: 234
                                                    maximumLineCount: 2
                                                    Style.class: "customerInfoValue"
                                                    elide: Text.ElideRight
                                                    text: params.customerInfo.creditType
                                                }
                                            }
                                        }
                                        RowLayout {
                                            spacing: 48
                                            ColumnLayout{
                                                Layout.fillWidth: true
                                                BasicLabel {
                                                    Style.class: "customerInfoLabel"
                                                    Tr { code: "customerNumberLabel"; defaultMessage: "Клиент" }
                                                }
                                                BasicLabel {
                                                    Style.class: "customerInfoValue"
                                                    text: params.customerInfo.clientNumber
                                                }
                                            }
                                            ColumnLayout {
                                                Layout.fillWidth: true
                                                BasicLabel {
                                                    Style.class: "customerInfoLabel"
                                                    Tr { code: "customerCardNumberLabel"; defaultMessage: "Номер карты" }
                                                }
                                                BasicLabel {
                                                    Style.class: "customerInfoValue"
                                                    text: params.customerInfo.cardNumber
                                                }
                                            }
                                            ColumnLayout {
                                                Layout.preferredWidth: 50
                                                BasicLabel {
                                                    visible: params.customerInfo.hasOwnProperty("edo")
                                                    Style.class: "customerInfoLabel"
                                                    Tr { code: "customerEdoLabel"; defaultMessage: "ЭДО" }
                                                }
                                                BasicLabel {
                                                    visible: params.customerInfo.hasOwnProperty("edo")
                                                    Style.class: "customerInfoValue"
                                                    text: params.customerInfo.edo ? "Да" : "Нет"
                                                }
                                            }
                                        }
                                    }
                                }
                                RowLayout {
                                    spacing: 16
                                    visible: contentInfo.expand ? 1 : 0
                                    InfoBoxBlue {
                                        contentItem: ColumnLayout{
                                            BasicLabel {
                                                Style.class: "customerInfoLabel"
                                                Tr { code: "customerCardHolderLabel"; defaultMessage: "Картодержатель" }
                                            }
                                            BasicLabel {
                                                Style.class: "customerInfoValue"
                                                text: params.customerInfo.cardholder
                                            }
                                        }
                                    }
                                    InfoBoxBlue {
                                        contentItem: ColumnLayout {
                                            BasicLabel {
                                                Style.class: "customerInfoLabel"
                                                Tr { code: "customerBranchLabel"; defaultMessage: "Вид деятельности" }
                                            }
                                            BasicLabel {
                                                Style.class: "customerInfoValue"
                                                text: params.customerInfo.branch
                                            }
                                        }
                                    }
                                    InfoBoxBlue {
                                        contentItem: ColumnLayout {
                                            BasicLabel {
                                                Style.class: "customerInfoLabel"
                                                Tr { code: "customerInnLabel"; defaultMessage: "ИНН" }
                                            }
                                            BasicLabel {
                                                Style.class: "customerInfoValue"
                                                text: params.customerInfo.inn
                                            }
                                        }
                                    }
                                }
                                RowLayout {
                                    spacing: 16
                                    visible: contentInfo.expand ? 1 : 0
                                    InfoBoxGreen {
                                        contentItem: ColumnLayout{
                                            BasicLabel {
                                                Style.class: "customerInfoLabel"
                                                Tr { code: "customerCreditLimitLabel"; defaultMessage: "Кредитный лимит" }
                                            }
                                            BasicLabel {
                                                Style.class: "customerInfoValue"
                                                text: params.customerInfo.hasOwnProperty("creditLimit")
                                                        ? Utils.formatMoneyItem(params.customerInfo.creditLimit) : ""
                                            }
                                            BasicLabel {
                                                Style.class: "customerInfoLabel"
                                                Tr { code: "customerCreditBalanceLabel"; defaultMessage: "Доступный остаток" }
                                            }
                                            BasicLabel {
                                                Style.class: "customerInfoValue"
                                                text: params.customerInfo.hasOwnProperty("creditBalance")
                                                        ? Utils.formatMoneyItem(params.customerInfo.creditBalance) : ""
                                            }
                                        }
                                    }
                                    InfoBoxCommon {
                                        Layout.preferredHeight: 166
                                        contentItem: ColumnLayout {
                                            BasicLabel {
                                                Layout.alignment: Qt.AlignTop
                                                Style.class: "customerInfoLabel"
                                                Tr { code: "customerBlockingLabel"; defaultMessage: "Блокировки" }
                                            }
                                            BasicLabel {
                                                visible: params.customerInfo.blockingCode.length !== 0
                                                Layout.alignment: Qt.AlignTop
                                                Layout.preferredWidth: 200
                                                Style.class: "customerInfoValue"
                                                maximumLineCount: 3
                                                elide: Text.ElideRight
                                                wrapMode: Text.WordWrap
                                                property string onlyCode
                                                property string codeWithReason
                                                Tr on onlyCode {
                                                    code: "customerInfoBlockingOnlyCode";
                                                    defaultMessage: "Код %1"
                                                    args: [ params.customerInfo.blockingCode ]
                                                }
                                                Tr on codeWithReason {
                                                    code: "customerInfoBlockingCodeWithReason";
                                                    defaultMessage: "Код %1. %2"
                                                    args: [ params.customerInfo.blockingCode, params.customerInfo.blockingReason ]
                                                }
                                                text: params.customerInfo.blockingReason.length > 0 ?  codeWithReason : onlyCode
                                            }
                                        }
                                    }
                                }
                                InfoBoxCommon {
                                    Layout.preferredHeight: 216
                                    visible: contentInfo.expand ? 1 : 0
                                    contentItem: ColumnLayout {
                                        BasicLabel {
                                            Style.class: "customerInfoLabel"
                                            Tr { code: "customerAddressLabel"; defaultMessage: "Адрес" }
                                        }
                                        BasicLabel {
                                            Style.class: "customerInfoValue"
                                            Layout.preferredWidth: parent.width
                                            maximumLineCount: 2
                                            elide: Text.ElideRight
                                            wrapMode: Text.WordWrap
                                            text: params.customerInfo.address
                                        }
                                        Item { Layout.preferredHeight: 24 }
                                        RowLayout {
                                            spacing: 40
                                            ColumnLayout{
                                                BasicLabel {
                                                    Style.class: "customerInfoLabel"
                                                    Tr { code: "customerPhoneLabel"; defaultMessage: "Телефон" }
                                                }
                                                BasicLabel {
                                                    Layout.preferredWidth: 230
                                                    Style.class: "customerInfoValue"
                                                    text: params.customerInfo.phone
                                                }
                                            }
                                            ColumnLayout {
                                                BasicLabel {
                                                    Style.class: "customerInfoLabel"
                                                    Tr { code: "customerEmailLabel"; defaultMessage: "E-mail" }
                                                }
                                                BasicLabel {
                                                    Layout.fillWidth: true
                                                    maximumLineCount: 1
                                                    elide: Text.ElideRight
                                                    Style.class: "customerInfoValue"
                                                    text: params.customerInfo.email
                                                }
                                            }
                                        }
                                    }
                                }
                                Item {
                                    Layout.fillWidth: true
                                    Layout.fillHeight: true
                                    RowLayout {
                                        anchors.centerIn: parent
                                        spacing: 0
                                        BasicLabel {
                                            property string collapseText
                                            property string expandText
                                            Tr on collapseText { code: "collapseText"; defaultMessage: "Свернуть" }
                                            Tr on expandText { code: "expandText"; defaultMessage: "Больше информации" }
                                            text: contentInfo.expand ? collapseText : expandText
                                            Style.class: "customerInfoExpandLabel"
                                        }
                                        Item { Layout.preferredWidth: 5 }
                                        Image {
                                            source: contentInfo.expand ? Style.parameter("icon.arrowUpGreen") : Style.parameter("icon.arrowDownGreen")
                                        }
                                    }
                                }
                            }
                            MouseArea {
                                anchors.fill: parent
                                onClicked:  {
                                    contentInfo.expand = !contentInfo.expand
                                    scrollBar.active = contentInfo.expand
                                }
                            }
                        }
                        Item {
                            Layout.topMargin: 12
                            Layout.leftMargin: 24
                            Layout.fillWidth: true
                            Layout.fillHeight: true
                            Repeater {
                                model: InputMultiTextModel { fields: params.fields }
                                delegate: DelegateChooser {
                                    role: "type"
                                    DelegateChoice {
                                        roleValue: InputMultiTextModel.Choice
                                        delegate: ChoiceDelegate {}
                                    }
                                }
                            }
                        }
                    }
                    ScrollBar.vertical: ScrollBar {
                        id: scrollBar
                        Timer {
                            id: scrollTimer
                            interval: 2500
                            onTriggered: parent.active = false
                        }
                        onActiveChanged: scrollTimer.start()
                    }
                }
            }
        }
        // Правая часть
        Item {
            Layout.fillHeight: true
            Layout.preferredWidth: 304
            TopPanel {
                anchors { left: parent.left; top: parent.top; right: parent.right }
                anchors.leftMargin: 24
                hideUserName: true
            }
            ColumnLayout {
                anchors { left: parent.left; right: parent.right; bottom: parent.bottom; margins: 24 }
                spacing: 8
                BasicButton {
                    Style.class: root.showInfo ? "addCustomerOk" : "addCustomerCancel"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 68
                    property string closeText
                    property string cancelText
                    Tr on cancelText { code: "addCustomerCancelButton"; defaultMessage: "Отмена" }
                    Tr on closeText { code: "addCustomerCloseButton"; defaultMessage: "Закрыть" }
                    text: root.showInfo ? closeText : cancelText
                    onClicked: handler.sendAnswer({})
                }
                BasicButton {
                    visible: !root.showInfo
                    Style.class: "addCustomerOk"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 68
                    iconLayout.alignment: Qt.AlignTop | Qt.AlignRight
                    Tr { code: "addCustomerOkButton"; defaultMessage: "Далее" }
                    icon: Style.parameter("icon.actionPanel.arrow")
                    onClicked: accept()
                }
            }
        }
    }
    function accept() {
        let fields = Object.values(fieldsById)
        let result = {}
        for (let i = 0; i < fields.length; ++i) {
            switch (fields[i].fieldType) {
            case InputMultiTextModel.Choice:
                result[fields[i].objectName] = fields[i].choiceValue
                break
            }
        }
        handler.sendMultiTextAnswer(params.fields, result)
    }

    function needVisible(field) {
        if (field.fieldId === "purposePurchase")
            return true
        let fields = Object.values(fieldsById)
        for (let i = 0; i < fields.length; ++i) {
            if (fields[i].objectName === "purposePurchase")
                return fields[i].choiceValue === 0
        }
        return true
    }

    DialogHandler { id: handler }
}
