import QtQuick
import QtQuick.Controls
import QtQuick.Layouts

import "components"
import "./view"

import Artix.Handlers
import Artix.Models
import Artix.Core
import Artix.Gui

/**
 * @brief Форма открытого документа
 */
BasicView {
    id: root

    // Фильтр и ресивер для работы с клавиатурой
    property BasicTextField eventReceiver: inputField
    property FormEventFilter eventFilter: openDocumentEventFilter

    FormEventFilter {
        id: openDocumentEventFilter
        onKeyboardData: (data, rawData, source) => {
                            inputField.text = ""
                            handler.input(data, rawData, source)
                        }
        onOk: {
            if (inputField.text.length === 0)
                return
            let text = inputField.text
            inputField.text = ""
            handler.input(text, text, InputSource.Keyboard)
        }
        onCancel: {
            inputField.text = ""
            handler.escCommand()
        }
    }

    contentItem: RowLayout {
        spacing: 24
        // Левая часть - список позиций
        GroupBox {
            Style.class: "positionsListBackground"
            Layout.fillHeight: true
            Layout.preferredWidth: 456
            ColumnLayout {
                anchors.fill: parent
                spacing: 0
                // Информационные оповещения
                SingleNotification {
                    Layout.fillWidth: true
                    Layout.preferredHeight: 40
                    filter: Notification.Info
                    onClicked: topPanel.open()
                }
                // Шапка списка позиций
                PositionsListHeader {
                    id: positionListHeader
                    Layout.fillWidth: true
                    Layout.preferredHeight: 57
                    positionsCount: positions.model.positionsCount
                }
                // Список позиций
                PositionsList {
                    id: positions
                    clip: true
                    withModifiers: true
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                    scrollToEndOnItemAdded: true
                    onStorno: (position) => { handler.stornoPosition(position) }
                    onChangeQuantity: (position, value) => { handler.changeQuantity(position, value) }
                    onConsultantDelete: (position) => { handler.consultantDelete(position) }
                }
                // Дополнительная навигация
                ListNavigation {
                    target: positions
                    Layout.fillWidth: true
                    Layout.preferredHeight: 116
                    visible: Style.parameter("capacitiveTouchscreen") === "false"
                }
            }
        }
        // Правая часть - элементы управления
        Item {
            Layout.fillWidth: true
            Layout.fillHeight: true
            // Верхняя панель кассы
            TopPanel {
                id: topPanel
                anchors { left: parent.left; top: parent.top; right: parent.right }
            }
            ColumnLayout {
                anchors { left: parent.left; bottom: parent.bottom; right: parent.right; top: topPanel.bottom }
                anchors { rightMargin: 24; bottomMargin: 24; topMargin: errorNotification.visible ? 0 : 24 }
                spacing: 0
                // Критические оповещения
                SingleNotification {
                    id: errorNotification
                    Layout.fillWidth: true
                    Layout.preferredHeight: 40
                    filter: Notification.Error | Notification.Mode
                    onClicked: topPanel.open()
                }
                Item { Layout.preferredHeight: errorNotification.visible ? 4 : 0 }
                // Сумма чека/заголовок панели действий
                Item {
                    Layout.fillWidth: true
                    Layout.preferredHeight: 48
                    BasicLabel {
                        id: title
                        anchors.left: parent.left
                        anchors.bottom: parent.bottom
                        Style.class: "documentOpenTotalTitle"
                        property var defaultTr:
                            Utils.makeTr("documentOpenTitleTotal", "Сумма")
                        Tr { id: titleTr; tr: titleTr.defaultTr }
                    }
                    BasicLabel {
                        id: totalValue
                        anchors.right: parent.right
                        anchors.verticalCenter: parent.verticalCenter
                        Style.class: "documentOpenTotalValue"
                        text: Utils.formatMoneyItem(Session.document.sumb)
                        horizontalAlignment: Text.AlignRight
                    }
                }
                Item { Layout.preferredHeight: errorNotification.visible ? 8 : 24 }
                // Поле ввода
                BasicTextField {
                    id: inputField
                    property var defaultTr:
                        Utils.makeTr("documentOpenInputBarcodePlaceholder", "Введите код товара")
                    property var defaultHorizontalAlignment: Text.AlignRight
                    property var defaultInputMask: ""
                    property var defaultFixedCursorPosition: true

                    Layout.fillWidth: true
                    Layout.preferredHeight: 64
                    Style.class: "documentOpenInput"
                    padding: 12
                    verticalAlignment: Text.AlignVCenter
                    horizontalAlignment: defaultHorizontalAlignment
                    hideCursor: true
                    fixedCursorPosition: defaultFixedCursorPosition
                    Tr on placeholderText { id: inputFieldPlaceholderTr; tr: inputField.defaultTr }
                    onClicked: {
                        if (actionPanelStack.depth === 1)
                            actionPanelStack.push("./view/actionPanelExtensions/TmcActionPanel.qml")
                    }
                }
                Item { Layout.preferredHeight: errorNotification.visible ? 20 : 24 }
                // Панель действий
                StackView {
                    id: actionPanelStack
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                    initialItem: ActionPanel {
                        id: actionPanel
                        context: Context.DocumentOpen
                        onClicked: (actionCode) => { handler.processAction(actionCode) }
                    }
                    popEnter: null
                    popExit: null
                    pushEnter: null
                    pushExit: null
                    onCurrentItemChanged: {
                        if (actionPanelStack.depth === 1) {
                            root.eventFilter = openDocumentEventFilter
                            inputFieldPlaceholderTr.tr = inputField.defaultTr
                            titleTr.tr = title.defaultTr
                            totalValue.opacity = 1.0
                            inputField.horizontalAlignment = inputField.defaultHorizontalAlignment
                            inputField.inputMask = inputField.defaultInputMask
                            inputField.fixedCursorPosition = inputField.defaultFixedCursorPosition
                            return
                        }

                        if (currentItem.eventFilter)
                            root.eventFilter = currentItem.eventFilter
                        if (currentItem.inputFieldMask) {
                            inputField.validator = currentItem.inputFieldMask
                            inputField.text = ""
                        }
                        currentItem.text = Qt.binding(() => inputField.text)
                        if (currentItem.placeholder !== undefined)
                            inputFieldPlaceholderTr.tr = Qt.binding(() => currentItem.placeholder)
                        if (currentItem.title !== undefined) {
                            titleTr.tr = Qt.binding(() => currentItem.title)
                            totalValue.opacity = 0.0
                        }
                        currentItem.back?.connect(() => {
                                                      inputField.text = ""
                                                      actionPanelStack.pop()

                                                  })
                        currentItem.commit?.connect((action, args) => {
                                                        actionPanelStack.pop()
                                                        inputField.text = ""
                                                        handler.emitAction(action, args)
                                                   })
                        currentItem.command?.connect((command, args) => {
                                                         actionPanelStack.pop()
                                                         inputField.text = ""
                                                         handler.emitAction(command, args)
                                                     })
                        currentItem.input?.connect((data) => {
                                                       actionPanelStack.pop()
                                                       inputField.text = ""
                                                       handler.input(data, data, InputSource.Keyboard)
                                                   })
                    }
                }
            }
        }
    }

    // Коннектим сигналы из C++ движка
    Connections {
        id: connections
        target: engine

        // Показать панель ввода даты
        function onShowInputDatePanelExtension(action) {
            actionPanelStack.push("./view/actionPanelExtensions/DateActionPanel.qml")
            actionPanelStack.currentItem.action = action
            inputField.horizontalAlignment = actionPanelStack.currentItem.inputTextHorizontalAlignment
            inputField.fixedCursorPosition = actionPanelStack.currentItem.inputTextFixedCursorPosition
        }
    }

    // Обработчик событий
    DocumentOpenHandler {
        id: handler
        onPageTo: (page) => { actionPanel.pushPage(page) }
        onPageBack: actionPanel.popPage()
        onPanelExtension: (viewName, action, position) => {
                              actionPanelStack.push("./view/actionPanelExtensions/" + viewName + ".qml")
                              actionPanelStack.currentItem.action = action
                              if (actionPanelStack.currentItem.hasOwnProperty("getDefaultInputValue"))
                                inputField.text = actionPanelStack.currentItem.getDefaultInputValue()
                              if (actionPanelStack.currentItem.hasOwnProperty("position"))
                                actionPanelStack.currentItem.position = position
                              if (actionPanelStack.currentItem.hasOwnProperty("inputTextFixedCursorPosition"))
                                inputField.fixedCursorPosition = actionPanelStack.currentItem.inputTextFixedCursorPosition
                              if (actionPanelStack.currentItem.hasOwnProperty("inputMask"))
                                inputField.inputMask = actionPanelStack.currentItem.inputMask
                              if (actionPanelStack.currentItem.hasOwnProperty("inputTextHorizontalAlignment"))
                                inputField.horizontalAlignment = actionPanelStack.currentItem.inputTextHorizontalAlignment
                          }
    }
}
