import QtQuick
import QtQuick.Controls
import QtQuick.Layouts

import "components"
import "./view"

import Artix.Handlers
import Artix.Core
import Artix.Gui

/**
 * @brief Диалог ввода количества
 */
ModalDialog {
    id: root
    implicitWidth: keyboard.visible ? defaultWidth + keyboard.width :
            Math.max(defaultWidth, quantityControls.width + leftPadding + rightPadding)

    property BasicTextField eventReceiver: inputField
    property DialogEventFilter eventFilter: DialogEventFilter {
        id: eventFilter
        onKeyboardData: (data, rawData, source) => {
               inputField.text = ""
               handler.input(data, rawData, source)
        }
        onOk: {
            if (!inputField.acceptableInput)
                return
            inputField.value = inputField.text
            handler.sendAnswer(inputField.value, InputSource.Ui)
        }
        onCancel: handler.sendAnswer()
    }

    property bool fractional: params.fractional
    Component.onCompleted: {
        if (fractional) {
            inputField.forceActiveFocus()
            inputField.selectAll()
        }
    }

    property list<int> quantityControlsQuick: [] //Список значений для кнопок быстрого изменения количества

    contentItem: ColumnLayout {
        spacing: 40
        Layout.fillWidth: true

        RowLayout {
            ColumnLayout {
                spacing: 32
                BasicLabel {
                    id: message
                    Style.class: "inputQuantityMessage"
                    Layout.fillWidth: true
                    horizontalAlignment: Label.AlignHCenter
                    Tr { tr: params.message }
                    visible: !posInfo.visible
                }
                RowLayout {
                    id: posInfo
                    visible: params.name.length > 0
                    spacing: 24
                    Image {
                        id: posImage
                        Layout.preferredWidth: 164
                        Layout.preferredHeight: 164
                        fillMode: Image.PreserveAspectFit
                        source: params.image.length > 0 ? params.image : Style.parameter("icon.goodsPlaceholder")
                        Component.onCompleted: mainWindow.scaleImage(this)
                    }
                    ColumnLayout {
                        Layout.topMargin: 24
                        Layout.bottomMargin: 24
                        spacing: 20
                        BasicLabel {
                            Style.class: "inputQuantityName"
                            Layout.fillWidth: true
                            Layout.maximumHeight: 96 - posPrice.height
                            text: params.name
                            elide: Text.ElideRight
                        }
                        BasicLabel {
                            id: posPrice
                            Style.class: "inputQuantityPrice"
                            Layout.fillWidth: true
                            text: Utils.formatMoneyItem(params.price)
                        }
                    }
                }
                ColumnLayout {
                    id: quantityControls
                    Layout.alignment: Qt.AlignHCenter
                    spacing: 24
                    RowLayout {
                        Layout.alignment: Qt.AlignHCenter
                        spacing: 16
                        BasicButton {
                            visible: !keyboard.visible
                            Layout.preferredWidth: 64
                            Layout.preferredHeight: 56
                            Style.class: "inputQuantityChange"
                            icon: Style.parameter("icon.minus")
                            onClicked: inputField.value -= 1
                            enabled: inputField.value >= 1
                        }
                        BasicTextField {
                            id: inputField
                            Style.class: "inputQuantityTextField"
                            inputMethodHints: Qt.ImhNoPredictiveText | Qt.ImhDigitsOnly
                            echoMode: TextInput.Normal
                            Layout.preferredWidth: root.fractional ? 288 : 144
                            Layout.preferredHeight: 56
                            horizontalAlignment: Text.AlignHCenter
                            hideCursor: true
                            fixedCursorPosition: true
                            enabled: root.fractional
                            validator: root.fractional ? dv : iv
                            RegularExpressionValidator {
                                id: dv
                                regularExpression: Utils.getRegExpForDecimals(5, 3)
                            }
                            IntValidator { id: iv; bottom: 0; top: 999999 }

                            property real value: params.defaultValue
                            text: root.fractional ? Utils.weightStr(value) : value.toFixed(0)
                        }
                        BasicButton {
                            visible: !keyboard.visible
                            Layout.preferredWidth: 64
                            Layout.preferredHeight: 56
                            Style.class: "inputQuantityChange"
                            icon: Style.parameter("icon.plus")
                            onClicked: inputField.value += 1
                        }
                    }

                    RowLayout {
                        spacing: 24
                        Layout.alignment: Qt.AlignHCenter
                        visible: !keyboard.visible && root.quantityControlsQuick.length > 0

                        Repeater {
                            model: root.quantityControlsQuick
                            BasicButton {
                                required property int modelData
                                padding: 0
                                Layout.preferredWidth: 72
                                Layout.preferredHeight: 72
                                radius: 36
                                Style.class: "inputQuantityChangeQuick"
                                text: modelData > 0 ? "+" + modelData : modelData
                                textLayout: LayoutProperty {
                                    alignment: Qt.AlignHCenter | Qt.AlignVCenter
                                }
                                enabled: inputField.value >= -modelData
                                onClicked: inputField.value += modelData
                            }
                        }
                    }
                }
            }
            InlineKeyboard {
                id: keyboard
                visible: root.fractional
                Layout.preferredHeight: 252
                Layout.alignment: Qt.AlignHCenter
                width: 256
                layoutType: "digits"
            }
        }

        RowLayout {
            Layout.alignment: Qt.AlignHCenter
            spacing: 16
            BasicButton {
                Style.class: "inputQuantityOk"
                Layout.preferredWidth: 200
                Layout.preferredHeight: 56
                textLayout.alignment: Qt.AlignHCenter | Qt.AlignVCenter
                Tr { tr: params.okButtonText }
                onClicked: {
                    if (!inputField.acceptableInput)
                        return
                    inputField.value = inputField.text
                    handler.sendAnswer(inputField.value, InputSource.Ui)
                }
            }
            BasicButton {
                Style.class: "inputQuantityCancel"
                Layout.preferredWidth: 200
                Layout.preferredHeight: 56
                textLayout.alignment: Qt.AlignHCenter | Qt.AlignVCenter
                Tr { tr: params.cancelButtonText }
                onClicked: handler.sendAnswer()
            }
        }
    }

    DialogHandler { id: handler }
}
