import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import Qt.labs.qmlmodels

import "components"
import "./view"

import Artix.Handlers
import Artix.Core
import Artix.Gui
import Artix.Models

/**
 * @brief Форма тестирования устройств ввода
 */
FullscreenDialog {

    function addKeyInfo(keyPressedInfo) {
        dataModel.addKey(keyPressedInfo)
        tableView.positionViewAtRow(tableView.rows - 1, TableView.AlignBottom)
    }

    property TestInputDevicesEventFilter eventFilter: TestInputDevicesEventFilter {
        onKeyPress: (keyPressedInfo) => addKeyInfo(keyPressedInfo)
        onCancel: () => handler.sendAnswer()
    }

    TestInputDevicesModel { id: dataModel }

    Connections {
        target: ActionQueueController
        function onInputData(data, rawData, source) {
            let kpi = KeyPressedInfo.create()
            kpi.inputDevice = InputSource.sourceName(source, false).ui()
            kpi.description = data
            addKeyInfo(kpi)
        }
    }

    Connections {
        target: KeyboardKeyWatcher
        function onKeyPress(keyPressedInfo) {
            addKeyInfo(keyPressedInfo)
        }
    }

    contentItem: RowLayout {
        spacing: 0
        Rectangle {
            id: contentRect
            Layout.fillWidth: true
            Layout.fillHeight: true
            color: Style.parameter("color.modalBackground")
            ColumnLayout {
                anchors { fill: parent; topMargin: 16 }
                spacing: 0
                BasicLabel {
                    Style.class: "testInputDevicesTitle"
                    Layout.fillWidth: true
                    Layout.leftMargin: 24
                    Layout.rightMargin: 24
                    Tr { code: "testInputDevicesFormTitle"; defaultMessage: "Тест устройств ввода" }
                }
                Item { Layout.preferredHeight: 24 }
                HorizontalHeaderView {
                    Layout.fillWidth: true
                    model: dataModel
                    clip: true
                    resizableColumns: false
                    interactive: false
                    columnWidthProvider: function (column) {
                        if (column === 2 || column === 3)
                            return 0
                        if (column === 0)
                            return width / 3
                        return width / 3 * 2
                    }
                    delegate: BasicLabel {
                        Style.class: "testInputDevicesTableTitle"
                        padding: 24
                        topPadding: 8
                        bottomPadding: 8
                        Tr { tr: decoration }
                    }
                    Line {
                        anchors.bottom: parent.bottom
                        strokeWidth: 2
                        width: parent.width
                        color: Style.parameter("color.grayDark")
                    }
                }
                TableView {
                    id: tableView
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                    columnWidthProvider: function (column) {
                        if (column === 2 || column === 3)
                            return 0
                        if (column === 0)
                            return width / 3
                        return width / 3 * 2
                    }
                    clip: true
                    contentWidth: width
                    resizableColumns: false
                    model: dataModel

                    // При масштабировании, списки могут отображаться с артефактами.
                    // Рендерим с мультисемплированием в нужном разрешении. Не знаю за счет чего, но проблему это решило.
                    layer {
                        enabled: mainWindow.multisampling
                        samples: 2
                        textureSize: Qt.size(width * mainWindow.scaleFactor, height * mainWindow.scaleFactor)
                    }

                    delegate: Control {
                        padding: 24
                        contentItem: BasicLabel {
                            Style.class: "testInputDevicesTableValue"
                            text: display
                        }
                        Line {
                            anchors.bottom: parent.bottom
                            strokeWidth: 2
                            width: parent.width
                            color: Style.parameter("color.grayBorder")
                        }
                    }
                    ScrollBar.vertical: ScrollBar {
                        id: scrollBar
                        Timer {
                            id: scrollTimer
                            interval: 2500
                            onTriggered: parent.active = false
                        }
                        onActiveChanged: scrollTimer.start()
                    }
                }
            }
        }
        Item {
            Layout.fillHeight: true
            Layout.preferredWidth: 304
            TopPanel {
                anchors { left: parent.left; top: parent.top; right: parent.right }
                anchors.leftMargin: 24
                hideUserName: true
            }
            ColumnLayout {
                anchors { left: parent.left; right: parent.right; bottom: parent.bottom; margins: 24 }
                spacing: 8
                BasicButton {
                    Style.class: "testInputDevicesCancel"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 68
                    Tr { code: "backButton"; defaultMessage: "Назад" }
                    onClicked: handler.sendAnswer()
                }
            }
        }
    }

    DialogHandler { id: handler }
}
