import QtQuick
import QtQuick.Controls
import QtQuick.Layouts

import Artix.Gui
import Artix.Models

/**
 * @brief Плиточное меню
 * @param model модель данных
 * @param delegate делегат элемента страницы меню
 * @param rows количество строк в меню
 * @param columns количество столбцов в меню
 * @param pageIndex индекс текущей страницы
 * @param pageCount количество страниц
 * @param spacing интервалы между кнопками
 * @todo При переключении currentIndex кнопками не происходит свайпа страницы. Пофиксить.
 **/
Item {
    id: root
    required property QAbstractItemModel model          ///< модель данных
    required property Component delegate                ///< делегат страницы меню
    property int columns: 3                             ///< количество столбцов в меню
    property int rows: 3                                ///< количество строк в меню
    property alias pageIndex: swipeView.currentIndex    ///< индекс текущей страницы
    readonly property int pageCount: pageRepeater.model ///< количество страниц
    property int spacing: 20                            ///< интервалы между кнопками
    function setCurrentPage(index) {
        // Установка -1 ломает анимацию перехода между страницами, но она нужна из-за бага.
        // Без -1, когда обновляется модель, преключение страниц по нажатию на точки пререстает работать
        swipeView.setCurrentIndex(-1)
        swipeView.setCurrentIndex(index)
    }

    // Необходимо устанавливать координаты и размеры делегата используя cellWidth, cellHeight, cellX и cellY
    readonly property real cellWidth: (swipeView.width - spacing * (columns - 1)) / columns
    readonly property real cellHeight: (swipeView.height - spacing * (rows - 1)) / rows
    function cellX(index) {
        var column = index % columns
        return column * (cellWidth + spacing)
    }
    function cellY(index) {
        var row = Math.trunc(index / columns)
        return row * (cellHeight + spacing)
    }
    Connections {
        target: model
        function onLayoutAboutToBeChanged() {
            // Выключаем анимацию перелистывания при обновлении модели
            swipeView.animationDuration = swipeView.contentItem.highlightMoveDuration
            swipeView.contentItem.highlightMoveDuration = 0
        }
        function onLayoutChanged() {
            // Считаем количество страниц, при обновлении модели
            pageRepeater.model = Math.ceil(root.model.rowCount() / (root.columns * root.rows))
            // Включаем анимацию перелистывания при обновлении модели
            swipeView.contentItem.highlightMoveDuration = swipeView.animationDuration
        }
    }
    ColumnLayout {
        anchors.fill: parent
        spacing: 24
        SwipeView {
            property int animationDuration: 0
            Layout.fillWidth: true
            Layout.fillHeight: true
            id: swipeView
            clip: true
            spacing: root.spacing

            // Создаем страницы для swipe меню с помощью повторителя
            Repeater {
                id: pageRepeater
                model: 0
                // Страница меню
                delegate: Item {
                    id: page
                    required property int index
                    // Создаем ячейки на основе данных, полученных из модели
                    Repeater {
                        // Используем proxy модель, чтобы выделить данные для конкретной страницы меню
                        model: PageFilterProxyModel {
                            sourceModel: root.model
                            pageIndex: page.index
                            pageElementCount: root.columns * root.rows
                        }
                        delegate: root.delegate
                    }
                }
            }
        }
        // Индикатор текущей страницы меню
        PageIndicator {
            id: indicator
            count: swipeView.count
            currentIndex: swipeView.currentIndex
            Layout.preferredHeight: 10
            Layout.alignment: Qt.AlignHCenter | Qt.AlignVCenter
            spacing: 16
            visible: count > 1
            delegate: Rectangle {
                required property int index
                width: 10; height: width
                radius: width / 2
                color: index === indicator.currentIndex ? Style.parameter("color.greenDark")
                                                        : Style.parameter("color.gray")
                TapHandler { onTapped: { root.setCurrentPage(index) } }
            }
        }
    }
}
